@@ -92,14 +92,24 @@
     #define FLREG flw
     #define FSREG fsw
     #define FREGWIDTH 4
-    #if XLEN==64
-        #define SIGALIGN 8
-    #else
-        #define SIGALIGN 4
-    #endif
   #endif
 #endif
 
+#if FLEN>XLEN
+    #define SIGALIGN FREGWIDTH
+#else
+    #define SIGALIGN REGWIDTH
+#endif
+
+
+#if SIGALIGN==8
+  #define CANARY \
+      .dword 0x6F5CA309E7D4B281
+#else
+  #define CANARY \
+      .word 0x6F5CA309 
+#endif
+
 #define MMODE_SIG 3
 #define RLENG (REGWIDTH<<3)
 
@@ -932,6 +942,23 @@ RVTEST_SIGUPD_F(swreg,destreg,flagreg)
     csrr DEST_REG,ADDRESS;\
     RVTEST_SIGUPD(BASE_REG,DEST_REG,OFFSET)
 
+#define WRITE_TO_CSR_FIELD_W_MASK(ADDRESS,RESTORE_REG,TEMP_REG1,TEMP_REG2,MASK_VAL,VAL) \
+  LI(TEMP_REG1,VAL);\
+  LI(TEMP_REG2,MASK_VAL);\
+  and TEMP_REG1,TEMP_REG1,TEMP_REG2;\
+  csrr RESTORE_REG,ADDRESS;\
+  not TEMP_REG2,TEMP_REG2;\
+  and RESTORE_REG,RESTORE_REG,TEMP_REG2;\
+  or TEMP_REG1,TEMP_REG1,RESTORE_REG;\
+  csrrw RESTORE_REG,ADDRESS,TEMP_REG1;\
+
+#define READ_CSR_REG_AND_UPD_SIG(ADDRESS,DEST_REG,OFFSET,BASE_REG) \
+  csrr DEST_REG,ADDRESS;\
+  RVTEST_SIGUPD(BASE_REG,DEST_REG,OFFSET)
+
+#define RESTORE_CSR_REG(ADDRESS,RESTORE_REG) \
+  csrw ADDRESS,RESTORE_REG;
+
 
 #define TEST_CASE(testreg, destreg, correctval, swreg, offset, code... ) \
     code; \
